#include "StdAfx.h"
#include "WarIfServer.h"   // class implemented
#include "WarLogWin32NtEventHandler.h"
#include "WarLogEngine.h"
#include "WarLogIoStreamHandler.h"
#include "WarLogFileHandler.h"
#include "WarFile.h"
#include "WarIfLogModule.h"
#include "WarIfRoot.h"
#include "WarIfSite.h"
#include "WarOptionList.h"
#include "WarOptionWin32.h"


using namespace std;

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

WarIfServer::WarIfServer(war_if_root_ptr_t& rootInterface) :
mRootPtr(rootInterface)
{
    mName = rootInterface->GetRegKey().GetPath().GetFilename().GetPath();
    mRegKey.Open(rootInterface->GetRegKey().GetRootKey(), 
        rootInterface->GetRegKey().GetPath());
}// WarIfServer

WarIfServer::~WarIfServer()
{
}// ~WarIfServer


//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void WarIfServer::StartLogModules() const
        throw (WarException)
{
    strlist_t modules;

    EnumLogModules(modules, true);

    for(strlist_t::const_iterator P = modules.begin()
        ; P != modules.end()
        ; P++)
    {
        WarCollector<char> log_name = P->c_str();
        war_registrypath_t module_path;
        
        module_path << mRegKey.GetPath().GetPath() 
            << WAR_SYSSLASH << WAR_WINNT_REG_LOG_MODULES
            << WAR_SYSSLASH << *P;
        
        WarWin32Registry module_key;
        module_key.Open(mRegKey.GetRootKey(), module_path.GetPath());
        
        if (!module_key.GetIntValue(WAR_WINNT_REG_ENABLE, false))
            continue;
        
        war_regstr_t module_clsid 
            = module_key.GetStrValue(WAR_WINNT_REG_CLASSID, NULL, true);
        
        if (module_clsid == war_regstr_t(_T(WAR_CLSID_LOG_COUT)))
        {
            WarLogEngine::GetEngine().InstallHandler(
                new WarLogIoStreamHandler(cout, "cout"));
            
        } else if (module_clsid == war_regstr_t(_T(WAR_CLSID_LOG_FILE)))
        {
            war_file_ptr_t log_file;
            war_syspath_t log_path = module_key.GetStrValue(_T("File Name"));
            log_file = WarFile::Create(log_path);
            
            try
            {
                log_file->Open(WarFileEnums::F_APPEND 
                    | WarFileEnums::F_WRITE
                    | WarFileEnums::F_SHREAD
                    | WarFileEnums::F_CREATE
                    | WarFileEnums::F_SEQUENTIAL);
            }
            catch(WarException)
            {

                WAR_CERR << _T("Failed to open logfile ") << log_path.GetPath() << endl;
                continue;
            }
            
            WarLogEngine::GetEngine().InstallHandler(
                new WarLogFileHandler(log_file, "logfile"));
            
        } else if (module_clsid == war_regstr_t(_T(WAR_CLSID_LOG_NTEVLOG)))
        {
            WarLogEngine::GetEngine().InstallHandler(
                new WarLogWin32NtEventHandler("eventlog", 
                (const WarLogWin32NtEventHandler::str_t) module_key.GetStrValue(_T("Tag Name")).c_str()));
        }
        else
        {
            WAR_CERR << _T("Unknown log module: ") << *P << endl;
            continue;
        }
        
        WarCollector<char> events = module_key.GetStrValue(_T("Events")); 
        
        if (!events.GetValue().empty())
        {
            WarLogEngine::GetEngine().EnableEvent(log_name.GetValue().c_str(), 
                events.GetValue().c_str());
        }
    }
}

war_if_log_module_ptr_t WarIfServer::GetLogModule(war_ccsysstr_t logName) 
    const throw (WarException)
{
    WarIfLogModule *pnew_module = new WarIfLogModule;
    pnew_module->Open(mRegKey, logName);

    return pnew_module;
}

void WarIfServer::CwdToServerDir() throw(WarException)
{
    if (!::SetCurrentDirectory(
        mRegKey.GetStrValue(WAR_WINNT_STARTUP_PATH).c_str()))
    {
        WarSystemError sys_err;

        // We don't have the log-module yet
        WAR_CERR << _T("Failed to change to directory: ") 
            << WAR_WINNT_STARTUP_PATH 
            << endl;

        WarThrow(sys_err, NULL);
    }
}

void WarIfServer::LoadOptions() throw(WarException)
{
    WarWin32Registry reg_options;

    reg_options.Create(mRegKey.GetRootKey(), mRegKey.GetPath(WAR_WINNT_REG_OPTIONS));
    
    WarOptionList& global_opts = WarOptionList::GetGlobalOptions();
    reg_options >> global_opts;
}

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

void WarIfServer::EnumLogModules(strlist_t& logModuleList, 
                                 bool hideDisabled) const 
                                 throw (WarException)
{
    WarWin32Registry log_key;
    WarWin32Registry::keyname_list_t keys;

    log_key.Open(mRegKey.GetRootKey(), mRegKey.GetPath(WAR_WINNT_REG_LOG_MODULES));

    log_key.EnumKey(keys);

    for(WarWin32Registry::keyname_list_t::const_iterator P = keys.begin()
        ; P != keys.end()
        ; P++)
    {
        WarWin32Registry module_key;

        try
        {
            module_key.Open(log_key.GetRootKey(), log_key.GetPath(P->c_str()));

            // Bail out if there is no "Class ID" 
            module_key.GetStrValue(WAR_WINNT_REG_CLASSID, NULL, true);

            if (hideDisabled 
                && !module_key.GetIntValue(WAR_WINNT_REG_ENABLE, false))
                continue;

            // Add it
            logModuleList.push_back(*P);
        }
        catch(WarException) 
        {
            ;
        }
    }
}

war_ccsysstr_t WarIfServer::GetName() const
{
    return mName.c_str();
}

void WarIfServer::EnumSites(war_if_site_list_t& siteList,
        bool hideDisabled)  
        throw (WarException)
{
    WarWin32Registry sites_root_key;
    WarWin32Registry::keyname_list_t keys;

    sites_root_key.Open(mRegKey.GetRootKey(),
        mRegKey.GetPath(WAR_WINNT_REG_SITES));
    sites_root_key.EnumKey(keys);

     for(WarWin32Registry::keyname_list_t::const_iterator P = keys.begin()
        ; P != keys.end()
        ; P++)
    {
        WarWin32Registry site_key;

        try
        {
            site_key.Open(sites_root_key.GetRootKey(), 
                sites_root_key.GetPath(P->c_str()));

            // Bail out if there is no "Class ID" 
            war_regstr_t class_id = site_key.GetStrValue(
                WAR_WINNT_REG_CLASSID, NULL, true);

            if (class_id != war_regstr_t(_T(WAR_CLSID_SITE)))
                continue;

            if (hideDisabled 
                && !site_key.GetIntValue(WAR_WINNT_REG_ENABLE, false))
                continue;

            // Add it
            siteList.push_back(new WarIfSite(
                war_if_server_ptr_t(this), site_key));

        }
        catch(WarException) 
        {
            ;
        }
    }
}

/////////////////////////////// PROTECTED  ///////////////////////////////////


/////////////////////////////// PRIVATE    ///////////////////////////////////

