#include "StdAfx.h"
#include "WarOptionWin32.h"   // class implemented

#ifndef WAR_OPTION_LIST_H
#   include "WarOptionList.h"
#endif

#define AUTO_LOCK WarAutoLock my_lock((WarCriticalSection &)mLock);

/////////////////////////////// PUBLIC ///////////////////////////////////////

WarWin32Registry& operator << (WarWin32Registry& o, 
                               const WarOptionList& value)
{
    // The registry is updated as nodes are created/changed. No need to 
    // flush.
    return o;
}

WarWin32Registry& operator >> (WarWin32Registry& o, 
                               WarOptionList& ops)
{
    WarWin32Registry::keyname_list_t keys;
    o.EnumKey(keys);

    for(WarWin32Registry::keyname_list_t::const_iterator P = keys.begin()
        ; P != keys.end()
        ; P++)
    {
        war_option_ptr_t my_option_ptr = new WarOptionWin32(
            o, WarCollector<char>(P->c_str()).GetValue().c_str(), 
            NULL, NULL, NULL, WarOption::OT_INVALID);

        ops.insert(my_option_ptr);
    }

    return o;
}


//============================= LIFECYCLE ====================================

WarOptionWin32::WarOptionWin32(
          const WarWin32Registry& fatherKey,
          war_ccstr_t name, 
          war_ccstr_t value,
          war_ccstr_t defaultValue,
          war_ccstr_t description,
          WarOptionTypeE type)
          throw(WarException)
          : WarOption(name, NULL, NULL, NULL, type) 
{
    war_registrypath_t node_name;
    node_name << name;

    try
    {
        mKey.Open(fatherKey.GetRootKey(), 
            fatherKey.GetPath(node_name.GetPath()));

        DoSetType((WarOptionTypeE)mKey.GetIntValue(WAR_WINNT_REG_TYPE, OT_STRING));
    }
    catch(WarException)
    {
        mKey.Create(fatherKey.GetRootKey(), 
            fatherKey.GetPath(node_name.GetPath()));

        mKey.SetValue(WAR_WINNT_REG_TYPE, GetType());
    }
    
    if (value)
        SetValue(value);

    if (defaultValue)
        SetDefaultValue(defaultValue);

    if (description)
        SetDescription(description);
}

//============================= OPERATORS ====================================



//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================

const std::string WarOptionWin32::GetValue() const
throw(WarException)
{
    WarCollector<war_registrypath_t::path_char_t> my_str;
    
    try
    {
        AUTO_LOCK
        my_str = mKey.GetStrValue(WAR_WINNT_REG_VALUE, NULL, true);   
    }
    catch(WarException) 
    {
        return GetDefaultValue();
    }

    return WarCollector<char>(my_str.GetValue()).GetValue();
}

const int WarOptionWin32::GetIntValue() const
        throw(WarException)
{
    int int_val = 0;

    try
    {
        AUTO_LOCK
        return mKey.GetIntValue(WAR_WINNT_REG_VALUE, 0, true);   
    }
    catch(WarException) 
    {
        return GetDefaultIntValue();
    }
    
    return 0; // compiler food
}

const std::string WarOptionWin32::GetDefaultValue() const
throw(WarException)
{
    WarCollector<war_registrypath_t::path_char_t> my_str;
    
    AUTO_LOCK    
    my_str = mKey.GetStrValue(WAR_WINNT_REG_DEF_VALUE);

    return WarCollector<char>(my_str.GetValue()).GetValue();
}

const int WarOptionWin32::GetDefaultIntValue() const
throw(WarException)
{
    int int_val = 0;
    
    AUTO_LOCK
    return mKey.GetIntValue(WAR_WINNT_REG_DEF_VALUE, 0, true);   
}

const std::string WarOptionWin32::GetDescription() const
throw(WarException)
{
    WarCollector<war_registrypath_t::path_char_t> my_str;
    
    AUTO_LOCK    
    my_str = mKey.GetStrValue(WAR_WINNT_REG_DESCRIPTION);

    return WarCollector<char>(my_str.GetValue()).GetValue();
}

//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

void WarOptionWin32::DoSetValue(const buffer_t& value)
throw(WarException)
{
    AUTO_LOCK

    switch(GetType())
    {
        case OT_BOOL:
        case OT_UNSIGNED:
        case OT_INT:
            {
                int int_value = atoi(value.GetValue().c_str());
                mKey.SetValue(WAR_WINNT_REG_VALUE, int_value);
            }
            break;
        default:
            {
                WarCollector<war_registrypath_t::path_char_t> 
                    my_str = value.GetValue();
                mKey.SetValue(WAR_WINNT_REG_VALUE, my_str.GetValue());
            }
            break;
    }
}

void WarOptionWin32::DoSetDefaultValue(const buffer_t& value)
throw(WarException)
{
    AUTO_LOCK

    switch(GetType())
    {
        case OT_BOOL:
        case OT_UNSIGNED:
        case OT_INT:
            {
                int int_value = atoi(value.GetValue().c_str());
                mKey.SetValue(WAR_WINNT_REG_DEF_VALUE, int_value);
            }
            break;
        default:
            {
                WarCollector<war_registrypath_t::path_char_t> 
                    my_str = value.GetValue();
                mKey.SetValue(WAR_WINNT_REG_DEF_VALUE, my_str.GetValue());
            }
            break;
    }
}

void WarOptionWin32::DoSetDescription(const buffer_t& value)
throw(WarException)
{
    AUTO_LOCK

    WarCollector<war_registrypath_t::path_char_t> my_str = value.GetValue();
    mKey.SetValue(WAR_WINNT_REG_DESCRIPTION, my_str.GetValue());
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

