#include "StdAfx.h"
#include "WarSocketPlugins.h"   // class implemented

#ifndef WAR_SOCKET_H
#   include "WarSocket.h"
#endif

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

#ifndef WAR_TIMER_H
#   include "WarTimer.h"
#endif

#define AUTO_LOCK WarAutoLock MyLock(mLock);

/////////////////////////////// WarSocketPluginPreOnReceived ///////////////////////

void WarSocketPluginPreOnReceived::OnXmitted(data_t& data)
{
    mpSocket->OnReceived(data.first, data.second);
}

/////////////////////////////// WarSocketPluginPreOnSent ///////////////////////


void WarSocketPluginPreOnSent::OnXmitted(data_t& data)
{
    mpSocket->OnSent(data.first, data.second);
}


/////////////////////////////// WarSocketPluginPreOnXmit ///////////////////////

WarSocketPluginPreOnXmit::WarSocketPluginPreOnXmit(
    WarSocket *pSocket,
    const unsigned delayMs,
    const unsigned holdMs,
    const unsigned minCnt,
    const unsigned maxCnt,
    const unsigned minCps,
    const unsigned maxCps)
    : mpSocket(pSocket),
    mMaxCps(maxCps),
    mCps(delayMs, holdMs, minCnt, maxCnt, minCps, maxCps)

{

}

void WarSocketPluginPreOnXmit::OnProcess(const WarError& status,
                                         war_transfer_buffer_ptr_t& buffer)
{   
    // We will only add samples to the cps object when the
    // buffer is empty

    AUTO_LOCK

    WarLog cps_log(WARLOG_CPS, "WarSocketPluginPreOnXmit::OnProcess()");
    if (cps_log)
        cps_log << "Evaluating CPS rate. ";

    bool is_empty;

    if (true == (is_empty = mBuffers.empty()))
        mCps.Record(buffer->GetBytesUsed());

    if (!is_empty || mCps.IsOverLimit())
    {
        mBuffers.push_back(data_t(status, buffer));

        if (cps_log)
            cps_log << "Caching the request for delayed callback." << war_endl;

        if (is_empty)
            SetTimer();

        WarThrow(WarError(WAR_ERR_PLUGIN_DONE), "suspend");
    }
    
    if (cps_log)
        cps_log << "Continuing with default processing." << war_endl;
    
}

void WarSocketPluginPreOnXmit::SetTimer()
{
    if (!mTimerPtr.IsEmpty())
        return; // Timer is set

    mTimerPtr = new timer_event_t(this, mCps.GetNextCpsTime());

    WarLog cps_log(WARLOG_CPS, "WarSocketPluginPreOnXmit::SetTimer()");
    if (cps_log)
    {
        cps_log << "Suspending transfer "
            << mpSocket->GetSeqNumber()
            << " due to CPS limit. Current CPS is "
            << mCps.GetCps()
            << " and the limit is "
            << mMaxCps
            << ". Timer event data: "
            << mTimerPtr->Explain()
            << war_endl;
    }

    WarTimer::GetTimer().AddEvent((war_timer_event_ptr_t &)mTimerPtr);
}    

void WarSocketPluginPreOnXmit::OnTimer()
{
    data_t my_data;
    int cnt = 0;

    WarLog cps_log(WARLOG_CPS, "WarSocketPluginPreOnXmit::OnTimer()");

    AUTO_LOCK

    if (cps_log)
    {
         cps_log << "Resuming transfer "
            << mpSocket->GetSeqNumber()
            << " Current CPS is "
            << mCps.GetCps()
            << war_endl;
    }

    mTimerPtr = NULL;
    
    if (mBuffers.empty())
        return; // Nothing to do
    
    /* The top buffer may be sampled, the rest is not, and we have to
       do the sampling here, before OnSent() is called. We can not
       sample the first entry!.
    */
    
    while(!mBuffers.empty())
    {
        my_data = mBuffers.front();
        
        if (0 != cnt++)
        {
            mCps.Record(my_data.second->GetBytesUsed());
            if (mCps.IsOverLimit())
            {
                SetTimer();
                return;
            }
        }
        
        mBuffers.pop_front();

        try
        {
            OnXmitted(my_data);
        }
        catch(WarException& e)
        {
            WarLog err_log(WARLOG_ERROR, "WarSocketPluginPreOnXmit::OnTimer()");

            err_log << "Caught unexpected exception:"
                << e 
                << " Ignoring it!"
                << war_endl;
        }
#if WAR_CATCH_ALL
        catch(...)
        {
            WarLog err_log(WARLOG_ERROR, "WarSocketPluginPreOnXmit::OnTimer()");

            err_log << "Caught unknown exception. Ignoring it!"
                << war_endl;
        }
#endif // WAR_CATCH_ALL
    }

    if (cps_log)
    {
        cps_log << "The queue is clear. Proceeding with undelayed processing."
            << war_endl;
    }

#ifdef _DEBUG
    assert(mBuffers.empty());
#endif
}

