#include "StdAfx.h"
#include "WarWin32Registry.h"   // class implemented

#ifndef WAR_INCLUDED_SHLWAPI_H
#   define WAR_INCLUDED_SHLWAPI_h
#   include "shlwapi.h"
#endif

#ifndef WAR_INCLUDED_TCHAR_H
#   define WAR_INCLUDED_TCHAR_H
#   include <tchar.h>
#endif

#define DEFAULT_REGKEY_BUFLEN (1024 * 4 * sizeof(TCHAR))
#define DEFAULT_NAME_BUFFER_LEN (512)

/////////////////////////////// PUBLIC ///////////////////////////////////////

war_registrypath_t WarWin32Registry::mRootPath;

//============================= LIFECYCLE ====================================

WarWin32Registry::WarWin32Registry()
: mNodeKey(0),
mRootKey(0)
{
}

WarWin32Registry::~WarWin32Registry()
{
    Reset();
}

//============================= OPERATORS ====================================


void WarWin32Registry::Open(HKEY keyRoot, 
                              const war_registrypath_t& regPath, 
                              int regMode)
                              throw(WarException)
{
    LONG status = 0;

    Reset();

    mPath = regPath.IsEmpty() ? mRootPath : regPath;

    if ((status = ::RegOpenKeyEx(
        keyRoot,
        mPath.GetPath(),
        0,
        regMode,
        &mNodeKey)) != ERROR_SUCCESS)
    {
        WarError sys_err(WAR_ERR_SYSTEM_ERROR, status);
        if (status == ERROR_FILE_NOT_FOUND)
            sys_err = WAR_ERR_NAME_DONT_EXIST;
        WarThrow(sys_err, NULL);
    }

    mRootKey = keyRoot;
}

void WarWin32Registry::Create(HKEY keyRoot, 
        const war_registrypath_t& regPath,
        bool doFailIfNotExist)
        throw(WarException)
{
    DWORD key_disposition = 0;
    LONG status = 0;

    mPath = regPath.IsEmpty() ? mRootPath : regPath;

    Reset();

    if ((status = ::RegCreateKeyEx(
        keyRoot,
        mPath.GetPath(),
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_WRITE | KEY_READ,
        NULL,
        &mNodeKey,
        &key_disposition)) != ERROR_SUCCESS)
    {
        WarThrow(WarSystemError(), NULL);
    }

    if (doFailIfNotExist 
        && (REG_OPENED_EXISTING_KEY == key_disposition))
    {
        Reset();
        WarThrow(WarError(WAR_ERR_OBJECT_EXIST), NULL);
    }

    mRootKey = keyRoot;
}

war_regstr_t 
WarWin32Registry::GetStrValue(war_registrypath_ccstr_t valueName,
                           war_registrypath_ccstr_t defaultValue,
                           bool doFailIfNotExist) const
                           throw(WarException)
{
    war_regstr_t return_val;
    LONG status;
    char byte_buffer[DEFAULT_REGKEY_BUFLEN];
    DWORD dw_len = sizeof(byte_buffer), dw_type = 0;

    if ((status = ::RegQueryValueEx(mNodeKey, valueName, 
        0, &dw_type, (LPBYTE)byte_buffer, &dw_len)) != ERROR_SUCCESS)
    {
        if((ERROR_FILE_NOT_FOUND == status)
            && !doFailIfNotExist)
        {
            return defaultValue ? defaultValue : _T("");
        }
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
    }

    switch(dw_type)
    {
        //case REG_DWORD:
        case REG_DWORD_LITTLE_ENDIAN:
        case REG_DWORD_BIG_ENDIAN:
            {
                DWORD my_dw_value = *(DWORD *)byte_buffer;
                if (REG_DWORD_BIG_ENDIAN == dw_type)
                    my_dw_value = ::ntohl(my_dw_value);
                WarItoa(return_val, my_dw_value);
            }
            break;
        case REG_EXPAND_SZ:
            {
                char tmp_buffer[DEFAULT_REGKEY_BUFLEN];
                if (!ExpandEnvironmentStrings(
                    (TCHAR *)byte_buffer, 
                    (TCHAR *)tmp_buffer, 
                    DEFAULT_REGKEY_BUFLEN / sizeof(TCHAR)) )
                {
                    WarThrow(WarSystemError(), NULL);
                }
                return_val = (TCHAR *)tmp_buffer;
            }
            break;
        case REG_SZ:
            return_val = (TCHAR *)byte_buffer;
            break;
        case REG_NONE:
            break;
#if 0  //  REG_QWORD
        case REG_QWORD:
        //case REG_QWORD_LITTLE_ENDIAN:
            {
                QWORD my_dw_value = *(QWORD *)byte_buffer;
                WarItoa(return_val, my_dw_value);
            }
            break;
#endif // REG_QWORD
        default:
            WarThrow(WarError(WAR_ERR_INVALID_CASE_VALUE), NULL);
            break;
    }

    return return_val;
}

DWORD WarWin32Registry::GetIntValue(war_registrypath_ccstr_t valueName,
                               DWORD defaultValue,
                               bool doFailIfNotExist) const
                               throw(WarException)
{
    DWORD return_val;
    LONG status;
    char byte_buffer[DEFAULT_REGKEY_BUFLEN];
    DWORD dw_len = sizeof(byte_buffer), dw_type = 0;

    if ((status = ::RegQueryValueEx(mNodeKey, valueName, 
        0, &dw_type, (LPBYTE)byte_buffer, &dw_len)) != ERROR_SUCCESS)
    {
        if (doFailIfNotExist)
            WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
        return defaultValue;
    }

    switch(dw_type)
    {
        //case REG_DWORD:
        case REG_DWORD_LITTLE_ENDIAN:
        case REG_DWORD_BIG_ENDIAN:
            {
                DWORD my_dw_value = *(DWORD *)byte_buffer;
                if (REG_DWORD_BIG_ENDIAN == dw_type)
                    my_dw_value = ::ntohl(my_dw_value);
                return_val = my_dw_value;
            }
            break;
        default:
            WarThrow(WarError(WAR_ERR_INVALID_CASE_VALUE), NULL);
            break;
    }

    return return_val;
}

WarWin32Registry::regstr_list_t 
WarWin32Registry::GetStrListValue(war_registrypath_ccstr_t valueName,
        regstr_list_t& defaultValue,
        bool doFailIfNotExist) const 
        throw(WarException)
{
    regstr_list_t return_val;
    war_regstr_t str_val;
    LONG status;
    char byte_buffer[DEFAULT_REGKEY_BUFLEN];
    DWORD dw_len = sizeof(byte_buffer), dw_type = 0;

    if ((status = ::RegQueryValueEx(mNodeKey, valueName, 
        0, &dw_type, (LPBYTE)byte_buffer, &dw_len)) != ERROR_SUCCESS)
    {
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
    }

    switch(dw_type)
    {
        //case REG_DWORD:
        case REG_DWORD_LITTLE_ENDIAN:
        case REG_DWORD_BIG_ENDIAN:
            {
                DWORD my_dw_value = *(DWORD *)byte_buffer;
                if (REG_DWORD_BIG_ENDIAN == dw_type)
                    my_dw_value = ::ntohl(my_dw_value);
                WarItoa(str_val, my_dw_value);
                return_val.push_back(str_val);
            }
            break;
        case REG_EXPAND_SZ:
            {
                char tmp_buffer[DEFAULT_REGKEY_BUFLEN];
                if (!ExpandEnvironmentStrings(
                    (TCHAR *)byte_buffer, 
                    (TCHAR *)tmp_buffer, 
                    DEFAULT_REGKEY_BUFLEN / sizeof(TCHAR)) )
                {
                    WarThrow(WarSystemError(), NULL);
                }
                str_val = (TCHAR *)tmp_buffer;
                return_val.push_back(str_val);
            }
            break;
        case REG_SZ:
            str_val = (TCHAR *)byte_buffer;
            return_val.push_back(str_val);
            break;
        case REG_MULTI_SZ:
            {
                for(TCHAR *p = (TCHAR *)byte_buffer; *p; *p++)
                {
                    return_val.push_back(war_regstr_t(p));
                    p += return_val.back().size();
                }
            }
            break;
        case REG_NONE:
            break;
#if defined(REG_QWORD) && defined(HAVE_QWORD)
        case REG_QWORD:
        //case REG_QWORD_LITTLE_ENDIAN:
            {
                QWORD my_dw_value = *(QWORD *)byte_buffer;
                WarItoa(str_val, my_dw_value);
                return_val.push_back(str_val);
            }
            break;
#endif // REG_QWORD
        default:
            WarThrow(WarError(WAR_ERR_INVALID_CASE_VALUE), NULL);
            break;
    }

    return return_val;
}


void WarWin32Registry::SetValue(war_registrypath_ccstr_t valueName, 
                                const war_regstr_t& newValue) const
                                throw(WarException)
{
    LONG status = 0;

    if ((status = ::RegSetValueEx(mNodeKey, valueName, 0,
        REG_SZ, (CONST BYTE *)(newValue.c_str()),
        (newValue.size() +1) * sizeof(TCHAR))) != ERROR_SUCCESS)
    {
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
    }
}


void WarWin32Registry::SetValue(war_registrypath_ccstr_t valueName, 
                                DWORD newValue) const
                                throw(WarException)
{
    LONG status = 0;

    if ((status = ::RegSetValueEx(mNodeKey, valueName, 0,
        REG_DWORD, (CONST BYTE *)(&newValue),
        sizeof(DWORD))) != ERROR_SUCCESS)
    {
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
    }
}


void WarWin32Registry::DeleteKey(war_registrypath_ccstr_t keyName) 
{
    if (keyName == NULL)
    {
        WarWin32Registry my_key;
        Reset();

        my_key.Open(mRootKey, mPath.GetPathname().GetPath(), KEY_WRITE);
        
        my_key.DeleteKey(mPath.GetFilename().GetPath());
        return;
    }

    LONG status = ::SHDeleteKey(mNodeKey, keyName);
    if (status != ERROR_SUCCESS)
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
}


void WarWin32Registry::DeleteValue(war_registrypath_ccstr_t valueName)
{
    LONG status = ::RegDeleteValue(mNodeKey, valueName);
    if (status != ERROR_SUCCESS)
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
}


void WarWin32Registry::EnumKey(keyname_list_t& resultList)
{
    int indedx = 0;
    LONG status;
    TCHAR name_buffer[DEFAULT_NAME_BUFFER_LEN];
    DWORD name_len = DEFAULT_NAME_BUFFER_LEN, class_len = 0;

    while((status = ::RegEnumKeyEx(mNodeKey,
        indedx++,
        name_buffer,
        &name_len,
        NULL,
        NULL,
        &class_len,
        NULL)) == ERROR_SUCCESS)
    {
        resultList.push_back(war_regstr_t(name_buffer));
        name_len = DEFAULT_NAME_BUFFER_LEN;
        class_len = 0;
    }

    if (ERROR_NO_MORE_ITEMS != status)
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
}


void WarWin32Registry::EnumValue(keyname_list_t& resultList)
{
    int indedx = 0;
    LONG status;
    TCHAR name_buffer[DEFAULT_NAME_BUFFER_LEN];
    DWORD name_len = DEFAULT_NAME_BUFFER_LEN;

    while((status = ::RegEnumValue(mNodeKey,
        indedx++,
        name_buffer,
        &name_len,
        NULL,
        NULL,
        NULL,
        NULL)) == ERROR_SUCCESS)
    {
        resultList.push_back(war_regstr_t(name_buffer));
        name_len = DEFAULT_NAME_BUFFER_LEN;
    }

    if (ERROR_NO_MORE_ITEMS != status)
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
}


void WarWin32Registry::Flush()
{
}

void WarWin32Registry::Rename(war_registrypath_ccstr_t newName)
        throw(WarException)
{
    if (!newName || !*newName)
        WarThrow(WarError(WAR_ERR_INVALID_ARGUMENT), NULL);

    war_registrypath_t new_path = mPath.GetPathname().GetPath();

    new_path << WAR_SYSSLASH << newName;

    try
    {
        WarWin32Registry test_key;
        test_key.Open(mRootKey, new_path, KEY_READ);
        WarThrow(WarError(WAR_ERR_NAME_EXIST), NULL);
    }
    catch(WarException)
    {
    }

    WarWin32Registry new_key, father_key;

    father_key.Open(mRootKey, 
        mPath.GetPathname().GetPath(), 
        KEY_ALL_ACCESS);
    
    new_key.Create(mRootKey, new_path);

    DWORD status = SHCopyKey(father_key.mNodeKey, 
        mPath.GetFilename().GetPath(),
        new_key.mNodeKey,
        NULL);
    if (status != ERROR_SUCCESS)
    {
        new_key.DeleteKey();
        WarThrow(WarError(WAR_ERR_SYSTEM_ERROR, status), NULL);
    }

    DeleteKey();

    // Steal the new properties from new_key
    mNodeKey = new_key.mNodeKey;
    new_key.mNodeKey = NULL;
    mRootKey = new_key.mRootKey;
    new_key.mRootKey = NULL;
    mPath = new_key.mPath;
}

//============================= OPERATIONS ===================================
//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

const war_registrypath_t 
WarWin32Registry::GetPath(war_registrypath_ccstr_t SubKeyName) const
{
    war_registrypath_t my_path = mPath;
    if (SubKeyName)
        my_path << WAR_SYSSLASH << SubKeyName;

    return my_path;
}


/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

void WarWin32Registry::Reset()
{
    if (IsOpen())
        ::CloseHandle(mNodeKey);
    
    mNodeKey = NULL;
}


